@@ -1,7 +1,37 @@
-#![deny(warnings, missing_docs, missing_debug_implementations)]
-#![doc(html_root_url = "https://docs.rs/slotmap/0.4.3")]
+#![doc(html_root_url = "https://docs.rs/slotmap/1.0.5")]
 #![crate_name = "slotmap"]
-#![cfg_attr(feature = "unstable", feature(untagged_unions, try_reserve))]
+#![cfg_attr(all(nightly, feature = "unstable"), feature(try_reserve))]
+#![cfg_attr(all(not(test), not(feature = "std")), no_std)]
+#![cfg_attr(all(nightly, doc), feature(doc_cfg))]
+#![warn(
+    invalid_html_tags,
+    missing_debug_implementations,
+    trivial_casts,
+    trivial_numeric_casts,
+    unused_lifetimes,
+    unused_import_braces
+)]
+#![deny(missing_docs, unaligned_references)]
+#![cfg_attr(feature = "cargo-clippy", allow(renamed_and_removed_lints))]
+#![cfg_attr(feature = "cargo-clippy", deny(clippy, clippy_pedantic))]
+#![cfg_attr(
+    feature = "cargo-clippy",
+    allow(
+        // Style differences.
+        module_name_repetitions,
+        redundant_closure_for_method_calls,
+        unseparated_literal_suffix,
+
+        // I know what I'm doing and want these.
+        wildcard_imports,
+        inline_always,
+        cast_possible_truncation,
+        needless_pass_by_value,
+
+        // Very noisy.
+        missing_errors_doc,
+        must_use_candidate
+    ))]
 
 //! # slotmap
 //!
@@ -23,6 +53,8 @@
 //! stored in slot maps, without hashing required - it's direct indexing under
 //! the hood.
 //!
+//! The minimum required stable Rust version for this crate is 1.49.
+//!
 //! # Examples
 //!
 //! ```
@@ -46,7 +78,7 @@
 //! }
 //! ```
 //!
-//! # Serialization through [`serde`]
+//! # Serialization through [`serde`], [`no_std`] support and unstable features
 //!
 //! Both keys and the slot maps have full (de)seralization support through
 //! the [`serde`] library. A key remains valid for a slot map even after one or
@@ -57,20 +89,31 @@
 //! feature flag for `slotmap` in your `Cargo.toml`.
 //!
 //! ```text
-//! slotmap = { version = "...", features = ["serde"] }
+//! slotmap = { version = "1.0", features = ["serde"] }
 //! ```
 //!
-//! # Why not [`slab`]?
+//! This crate also supports [`no_std`] environments, but does require the
+//! [`alloc`] crate to be available. To enable this you have to disable the
+//! `std` feature that is enabled by default:
 //!
-//! Unlike [`slab`], the keys returned by [`SlotMap`] are versioned. This means
-//! that once a key is removed, it stays removed, even if the physical storage
-//! inside the slotmap is reused for new elements. The key is a
+//! ```text
+//! slotmap = { version = "1.0", default-features = false }
+//! ```
+//!
+//! Unfortunately [`SparseSecondaryMap`] is not available in [`no_std`], because
+//! it relies on [`HashMap`]. Finally the `unstable` feature can be defined to
+//! enable the parts of `slotmap` that only work on nightly Rust.
+//!
+//! # Why not index a [`Vec`], or use [`slab`], [`stable-vec`], etc?
+//!
+//! Those solutions either can not reclaim memory from deleted elements or
+//! suffer from the ABA problem. The keys returned by `slotmap` are versioned.
+//! This means that once a key is removed, it stays removed, even if the
+//! physical storage inside the slotmap is reused for new elements. The key is a
 //! permanently unique<sup>*</sup> reference to the inserted value. Despite
-//! supporting versioning, a [`SlotMap`] is not slower than [`slab`], by
-//! internally using carefully checked unsafe code. A [`HopSlotMap`]
-//! also provides faster iteration than [`slab`] does, and [`DenseSlotMap`] even
-//! faster still. Additionally, at the time of writing [`slab`] does not support
-//! serialization.
+//! supporting versioning, a [`SlotMap`] is often not (much) slower than the
+//! alternative, by internally using carefully checked unsafe code. Finally,
+//! `slotmap` simply has a lot of features that make your life easy.
 //!
 //! # Performance characteristics and implementation details
 //!
@@ -92,28 +135,29 @@
 //! for [`HopSlotMap`]. [`DenseSlotMap`] has an overhead of 8 bytes per element
 //! and 8 bytes per slot.
 //!
-//! # Choosing `SlotMap`, `HopSlotMap` or `DenseSlotMap`
+//! # Choosing [`SlotMap`], [`HopSlotMap`] or [`DenseSlotMap`]
 //!
-//! A [`SlotMap`] can never shrink the size of its underlying storage, because
-//! for each storage slot it must remember what the latest stored version was,
-//! even if the slot is empty now. This means that iteration can be slow as it
-//! must iterate over potentially a lot of empty slots.
+//! A [`SlotMap`] is the fastest for most operations, except iteration. It can
+//! never shrink the size of its underlying storage, because it must remember
+//! for each storage slot what the latest stored version was, even if the slot
+//! is empty now. This means that iteration can be slow as it must iterate over
+//! potentially a lot of empty slots.
 //!
 //! [`HopSlotMap`] solves this by maintaining more information on
 //! insertion/removal, allowing it to iterate only over filled slots by 'hopping
 //! over' contiguous blocks of vacant slots. This can give it significantly
 //! better iteration speed.  If you expect to iterate over all elements in a
-//! [`SlotMap`] a lot, choose [`HopSlotMap`]. The downside is that insertion and
-//! removal is roughly twice as slow. Random access is the same speed for both.
+//! [`SlotMap`] a lot, and potentially have a lot of deleted elements, choose
+//! [`HopSlotMap`]. The downside is that insertion and removal is roughly twice
+//! as slow. Random access is the same speed for both.
 //!
 //! [`DenseSlotMap`] goes even further and stores all elements on a contiguous
-//! block of memory. It uses two indirects per random access; the slots contain
-//! indices used to access the contiguous memory. This means random access is
-//! slower than both [`SlotMap`] and [`HopSlotMap`], but iteration is
-//! significantly faster. Finally, there is no trait requirement on the value
-//! type of a [`DenseSlotMap`], see [`Slottable`] for more details.
+//! block of memory. It uses two indirections per random access; the slots
+//! contain indices used to access the contiguous memory. This means random
+//! access is slower than both [`SlotMap`] and [`HopSlotMap`], but iteration is
+//! significantly faster, as fast as a normal [`Vec`].
 //!
-//! # Choosing `SecondaryMap` or `SparseSecondaryMap`
+//! # Choosing [`SecondaryMap`] or [`SparseSecondaryMap`]
 //!
 //! You want to associate extra data with objects stored in a slot map, so you
 //! use (multiple) secondary maps to map keys to that data.
@@ -137,10 +181,10 @@
 //! detected at runtime, so it can lead to a hard to find bug.
 //!
 //! To prevent this, slot maps allow you to specify what the type is of the key
-//! they return, as long as that type implements the [`Key`] trait. To aid with
-//! this, the [`new_key_type!`] macro is provided that builds such a type for
-//! you. The resulting type is exactly like [`DefaultKey`]. So instead of simply
-//! using `SlotMap<DefaultKey, Player>` you would use:
+//! they return. You can construct new key types using the [`new_key_type!`]
+//! macro. The resulting type behaves exactly like [`DefaultKey`], but is a
+//! distinct type. So instead of simply using `SlotMap<DefaultKey, Player>` you
+//! would use:
 //!
 //! ```
 //! # use slotmap::*;
@@ -150,23 +194,17 @@
 //! let sm: SlotMap<PlayerKey, Player> = SlotMap::with_key();
 //! ```
 //!
-//! [`Vec`]: https://doc.rust-lang.org/std/vec/struct.Vec.html
-//! [`BTreeMap`]: https://doc.rust-lang.org/std/collections/struct.BTreeMap.html
-//! [`HashMap`]: https://doc.rust-lang.org/std/collections/struct.HashMap.html
-//! [`SlotMap`]: struct.SlotMap.html
-//! [`HopSlotMap`]: hop/struct.HopSlotMap.html
-//! [`DenseSlotMap`]: dense/struct.DenseSlotMap.html
-//! [`SecondaryMap`]: secondary/struct.SecondaryMap.html
-//! [`SparseSecondaryMap`]: sparse_secondary/struct.SparseSecondaryMap.html
-//! [`Slottable`]: trait.Slottable.html
-//! [`Key`]: trait.Key.html
-//! [`new_key_type!`]: macro.new_key_type.html
+//! You can write code generic over any key type using the [`Key`] trait.
+//!
+//! [`Vec`]: std::vec::Vec
+//! [`BTreeMap`]: std::collections::BTreeMap
+//! [`HashMap`]: std::collections::HashMap
 //! [`serde`]: https://github.com/serde-rs/serde
-//! [`slab`]: https://github.com/carllerche/slab
-//! [`DefaultKey`]: struct.DefaultKey.html
+//! [`slab`]: https://crates.io/crates/slab
+//! [`stable-vec`]: https://crates.io/crates/stable-vec
+//! [`no_std`]: https://doc.rust-lang.org/1.7.0/book/no-stdlib.html
 
-#[cfg(feature = "serde")]
-extern crate serde;
+extern crate alloc;
 
 // So our macros can refer to these.
 #[cfg(feature = "serde")]
@@ -175,90 +213,50 @@ pub mod __impl {
     pub use serde::{Deserialize, Deserializer, Serialize, Serializer};
 }
 
-#[cfg(test)]
-#[macro_use]
-extern crate quickcheck;
-
-#[cfg(test)]
-extern crate serde_json;
-
-pub(crate) mod normal;
-pub use crate::normal::*;
-
+pub mod basic;
 pub mod dense;
+pub mod hop;
+pub mod secondary;
+#[cfg(feature = "std")]
+pub mod sparse_secondary;
+
+#[doc(inline)]
+pub use crate::basic::SlotMap;
+
+#[doc(inline)]
 pub use crate::dense::DenseSlotMap;
 
-pub mod hop;
+#[doc(inline)]
 pub use crate::hop::HopSlotMap;
 
-pub mod secondary;
+#[doc(inline)]
 pub use crate::secondary::SecondaryMap;
 
-pub mod sparse_secondary;
+#[cfg(feature = "std")]
+#[doc(inline)]
 pub use crate::sparse_secondary::SparseSecondaryMap;
 
-use std::fmt::{self, Debug, Formatter};
-use std::num::NonZeroU32;
+use core::fmt::{self, Debug, Formatter};
+use core::num::NonZeroU32;
 
-/// A trait for items that can go in a [`SlotMap`] or [`HopSlotMap`]. Due to
-/// current stable Rust restrictions a type must be [`Copy`] to be placed in one
-/// of those slot maps. This restriction does not apply to [`DenseSlotMap`],
-/// [`SecondaryMap`] or [`SparseSecondaryMap`]. It also does not apply if you
-/// use nightly Rust and enable the `unstable` feature for `slotmap` by editing
-/// your `Cargo.toml`:
-///
-/// ```text
-/// slotmap = { version = "...", features = ["unstable"] }
-/// ```
-///
-/// This trait should already be automatically implemented for any type that is
-/// slottable.
-///
-/// [`Copy`]: https://doc.rust-lang.org/std/marker/trait.Copy.html
-/// [`SecondaryMap`]: secondary/struct.SecondaryMap.html
-/// [`SparseSecondaryMap`]: sparse_secondary/struct.SparseSecondaryMap.html
-/// [`SlotMap`]: struct.SlotMap.html
-/// [`HopSlotMap`]: hop/struct.HopSlotMap.html
-/// [`DenseSlotMap`]: dense/struct.DenseSlotMap.html
-#[cfg(not(feature = "unstable"))]
-pub trait Slottable: Copy {}
-
-/// A trait for items that can go in a [`SlotMap`] or [`HopSlotMap`]. Due to
-/// current stable Rust restrictions a type must be [`Copy`] to be placed in one
-/// of those slot maps. This restriction does not apply to [`DenseSlotMap`],
-/// [`SecondaryMap`] or [`SparseSecondaryMap`]. It also does not apply if you
-/// use nightly Rust and enable the `unstable` feature for `slotmap` by editing
-/// your `Cargo.toml`:
-///
-/// ```text
-/// slotmap = { version = "...", features = ["unstable"] }
-/// ```
-///
-/// This trait should already be automatically implemented for any type that is
-/// slottable.
-///
-/// [`Copy`]: https://doc.rust-lang.org/std/marker/trait.Copy.html
-/// [`SecondaryMap`]: secondary/struct.SecondaryMap.html
-/// [`SparseSecondaryMap`]: sparse_secondary/struct.SparseSecondaryMap.html
-/// [`SlotMap`]: struct.SlotMap.html
-/// [`HopSlotMap`]: hop/struct.HopSlotMap.html
-/// [`DenseSlotMap`]: dense/struct.DenseSlotMap.html
-#[cfg(feature = "unstable")]
+// Keep Slottable for backwards compatibility, but warn about deprecation
+// and hide from documentation.
+#[doc(hidden)]
+#[deprecated(
+    since = "1.0.0",
+    note = "Slottable is not necessary anymore, slotmap now supports all types on stable."
+)]
 pub trait Slottable {}
 
-#[cfg(not(feature = "unstable"))]
-impl<T: Copy> Slottable for T {}
-
-#[cfg(feature = "unstable")]
+#[doc(hidden)]
+#[allow(deprecated)]
 impl<T> Slottable for T {}
 
 /// The actual data stored in a [`Key`].
 ///
-/// This implements `Ord` so keys can be stored in e.g. [`BTreeMap`], but the
-/// order of keys is unspecified.
-///
-/// [`Key`]: trait.Key.html
-/// [`BTreeMap`]: https://doc.rust-lang.org/std/collections/struct.BTreeMap.html
+/// This implements [`Ord`](std::cmp::Ord) so keys can be stored in e.g.
+/// [`BTreeMap`](std::collections::BTreeMap), but the order of keys is
+/// unspecified.
 #[derive(Clone, Copy, PartialEq, Eq, PartialOrd, Ord, Hash)]
 pub struct KeyData {
     idx: u32,
@@ -267,23 +265,25 @@ pub struct KeyData {
 
 impl KeyData {
     fn new(idx: u32, version: u32) -> Self {
+        debug_assert!(version > 0);
+
         Self {
             idx,
-            version: NonZeroU32::new(version).expect("KeyData constructed with zero version"),
+            version: unsafe { NonZeroU32::new_unchecked(version | 1) },
         }
     }
 
     fn null() -> Self {
-        Self::new(std::u32::MAX, 1)
+        Self::new(core::u32::MAX, 1)
     }
 
     fn is_null(self) -> bool {
-        self.idx == std::u32::MAX
+        self.idx == core::u32::MAX
     }
 
     /// Returns the key data as a 64-bit integer. No guarantees about its value
-    /// are made other than that passing it to `from_ffi` will return a key
-    /// equal to the original.
+    /// are made other than that passing it to [`from_ffi`](Self::from_ffi)
+    /// will return a key equal to the original.
     ///
     /// With this you can easily pass slot map keys as opaque handles to foreign
     /// code. After you get them back you can confidently use them in your slot
@@ -294,7 +294,7 @@ impl KeyData {
     /// that. If you are not doing FFI, you almost surely do not need this
     /// function.
     ///
-    /// [`serde`]: index.html#serialization-through-serde
+    /// [`serde`]: crate#serialization-through-serde-no_std-support-and-unstable-features
     pub fn as_ffi(self) -> u64 {
         (u64::from(self.version.get()) << 32) | u64::from(self.idx)
     }
@@ -326,20 +326,20 @@ impl Default for KeyData {
 /// non-sensical (and might panic in case of out-of-bounds).
 ///
 /// To prevent this, it is suggested to have a unique key type for each slot
-/// map. The easiest way to do this is through [`new_key_type!`], which
-/// makes a new type identical to [`DefaultKey`], just with a different name.
-///
-/// This trait is intended to be a thin wrapper around [`KeyData`], and all
-/// methods must behave exactly as if we're operating on a [`KeyData`] directly.
-/// The internal unsafe code relies on this, therefore this trait is `unsafe` to
-/// implement. It is strongly suggested to simply use [`new_key_type!`] instead
-/// of implementing this trait yourself.
-///
-///
-/// [`new_key_type!`]: macro.new_key_type.html
-/// [`DefaultKey`]: struct.DefaultKey.html
-/// [`KeyData`]: struct.KeyData.html
-pub unsafe trait Key: From<KeyData> + Into<KeyData> + Clone {
+/// map. You can create new key types using [`new_key_type!`], which makes a
+/// new type identical to [`DefaultKey`], just with a different name.
+pub trait Key:
+    From<KeyData>
+    + Copy
+    + Clone
+    + Default
+    + Eq
+    + PartialEq
+    + Ord
+    + PartialOrd
+    + core::hash::Hash
+    + core::fmt::Debug
+{
     /// Creates a new key that is always invalid and distinct from any non-null
     /// key. A null key can only be created through this method (or default
     /// initialization of keys made with [`new_key_type!`], which calls this
@@ -360,8 +360,6 @@ pub unsafe trait Key: From<KeyData> + Into<KeyData> + Clone {
     /// assert!(k != nk);
     /// assert_eq!(sm.get(nk), None);
     /// ```
-    ///
-    /// [`new_key_type!`]: macro.new_key_type.html
     fn null() -> Self {
         KeyData::null().into()
     }
@@ -379,19 +377,30 @@ pub unsafe trait Key: From<KeyData> + Into<KeyData> + Clone {
     /// assert_eq!(a, b);
     /// assert!(a.is_null());
     /// ```
-    fn is_null(self) -> bool {
-        self.into().is_null()
+    fn is_null(&self) -> bool {
+        self.data().is_null()
     }
+
+    /// Gets the [`KeyData`] stored in this key.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// # use slotmap::*;
+    /// new_key_type! { struct MyKey; }
+    /// let dk = DefaultKey::null();
+    /// let mk = MyKey::null();
+    /// assert_eq!(dk.data(), mk.data());
+    /// ```
+    fn data(&self) -> KeyData;
 }
 
-/// A helper macro to conveniently create new key types. If you use a new key
-/// type for each slot map you create you can entirely prevent using the wrong
-/// key on the wrong slot map.
+/// A helper macro to create new key types. If you use a new key type for each
+/// slot map you create you can entirely prevent using the wrong key on the
+/// wrong slot map.
 ///
-/// The type constructed by this macro is identical to [`DefaultKey`], just with
-/// a different name.
-///
-/// [`DefaultKey`]: struct.DefaultKey.html
+/// The type constructed by this macro is defined exactly as [`DefaultKey`],
+/// but is a distinct type for the type checker and does not implicitly convert.
 ///
 /// # Examples
 ///
@@ -399,18 +408,30 @@ pub unsafe trait Key: From<KeyData> + Into<KeyData> + Clone {
 /// # extern crate slotmap;
 /// # use slotmap::*;
 /// new_key_type! {
-///     struct EntityKey;
+///     // A private key type.
+///     struct RocketKey;
 ///
-///     /// Key for the Player slot map.
-///     pub struct PlayerKey;
+///     // A public key type with a doc comment.
+///     /// Key for the user slot map.
+///     pub struct UserKey;
 /// }
 ///
 /// fn main() {
-///     let mut players = SlotMap::with_key();
-///     let mut entities: SlotMap<EntityKey, (f64, f64)> = SlotMap::with_key();
-///     let bob: PlayerKey = players.insert("bobby");
-///     // Now this is a type error because entities.get expects an EntityKey:
-///     // entities.get(bob);
+///     let mut users = SlotMap::with_key();
+///     let mut rockets = SlotMap::with_key();
+///     let bob: UserKey = users.insert("bobby");
+///     let apollo: RocketKey = rockets.insert("apollo");
+///     // Now this is a type error because rockets.get expects an RocketKey:
+///     // rockets.get(bob);
+///
+///     // If for some reason you do end up needing to convert (e.g. storing
+///     // keys of multiple slot maps in the same data structure without
+///     // boxing), you can use KeyData as an intermediate representation. This
+///     // does mean that once again you are responsible for not using the wrong
+///     // key on the wrong slot map.
+///     let keys = vec![bob.data(), apollo.data()];
+///     println!("{} likes rocket {}",
+///              users[keys[0].into()], rockets[keys[1].into()]);
 /// }
 /// ```
 #[macro_export(local_inner_macros)]
@@ -429,14 +450,12 @@ macro_rules! new_key_type {
             }
         }
 
-        impl From<$name> for $crate::KeyData {
-            fn from(k: $name) -> Self {
-                k.0
+        impl $crate::Key for $name {
+            fn data(&self) -> $crate::KeyData {
+                self.0
             }
         }
 
-        unsafe impl $crate::Key for $name { }
-
         $crate::__serialize_key!($name);
 
         $crate::new_key_type!($($rest)*);
@@ -455,7 +474,7 @@ macro_rules! __serialize_key {
             where
                 S: $crate::__impl::Serializer,
             {
-                $crate::KeyData::from(*self).serialize(serializer)
+                $crate::Key::data(self).serialize(serializer)
             }
         }
 
@@ -524,7 +543,7 @@ mod serialize {
             let mut ser_key: SerKey = Deserialize::deserialize(deserializer)?;
 
             // Ensure a.is_null() && b.is_null() implies a == b.
-            if ser_key.idx == std::u32::MAX {
+            if ser_key.idx == core::u32::MAX {
                 ser_key.version = 1;
             }
 
@@ -536,21 +555,36 @@ mod serialize {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
+    // Intentionally no `use super::*;` because we want to test macro expansion
+    // in the *users* scope, which might not have that.
+    #[test]
+    fn macro_expansion() {
+        use super::new_key_type;
+
+        new_key_type! {
+            struct A;
+            pub(crate) struct B;
+            pub struct C;
+        }
+    }
 
     #[test]
     fn check_is_older_version() {
+        use super::*;
+
         let is_older = |a, b| is_older_version(a, b);
         assert!(!is_older(42, 42));
         assert!(is_older(0, 1));
         assert!(is_older(0, 1 << 31));
         assert!(!is_older(0, (1 << 31) + 1));
-        assert!(is_older((-1i32) as u32, 0));
+        assert!(is_older(u32::MAX, 0));
     }
 
     #[cfg(feature = "serde")]
     #[test]
     fn key_serde() {
+        use super::*;
+
         // Check round-trip through serde.
         let mut sm = SlotMap::new();
         let k = sm.insert(42);
